function [b,a]=spbiln(d,c)
% [b,a]=spbiln(d,c)
% Converts H(s) with weights d,c to H(z) with weights a,b
% via bilinear transform. See Section 7.2 of Signal Proc. Algorithms.
% H(s)=D(s)/C(s); H(z)=B(z)/A(z).
%           Analog transfer function       Digital  transfer function
%
%                  d(L)*s^L+...+d(0)            b(1)+...+b(L+1)*z^(-L)
%           H(s) = -----------------     H(z) = ----------------------
%                  c(L)*s^L+...+c(0)              1+...+a(L+1)*z^(-L)
%
%           H(s) is assumed to be pre-scaled and pre-warped.
%
% Inputs:
%    d      = D(s) coefficient row vector.
%    c      = C(s) coefficient row vector.
% Outputs:  
%    b      = B(z) coefficient row vector.
%    a      = A(z) coefficient row vector with a(1)=1.

if(length(c) < length(d)),
   c=[c, zeros(1,length(d)-length(c))];
elseif(length(d) < length(c)),
   d=[d, zeros(1,length(c)-length(d))];
end
Ln=length(c)-1;
work=0*ones(Ln+1,Ln+1);
ierror=1;
i=Ln;
while i>=0,
   if (c(i+1)~=0)|(d(i+1)~=0)
      ierror=0;
      L=i;
      break
   end
   i=i-1;
end
if ierror==1
   error('SPBILN: c and d weights are all zeros.');
end
work(1,:)=ones(1,Ln+1);
tmp=1;
for i=1:L
   tmp=tmp*(L-i+1)/i;
   work(i+1,1)=tmp;
end
for i=1:L,
   for j=1:L,
      work(i+1,j+1)=work(i+1,j)-work(i,j+1)-work(i,j);
   end
end
for i=L:-1:0,
   b(i+1)=sum(work(i+1,(1:L+1)).*d(1:L+1));
   a(i+1)=sum(work(i+1,(1:L+1)).*c(1:L+1));
end
scale=a(1);
if scale==0,
   error('SPBILN: Computed value of a(1)=0.');
end
b=b/scale;
a=a/scale;
if L>=Ln
   return
else
   b(L+2:Ln+1)=zeros(1,Ln-L);
   a(L+2:Ln+1)=zeros(1,Ln-L);
   return
end
