function [d,c]=spcbii(L,k,ws,lamda)
% [d,c]=spcbii(L,k,ws,lamda)
% Generates kth section coefficients for Lth order normalized
% lowpass Chebyshev type II analog filter.  ws=stopband edge
% defined in Eq. 7.18, >1.  lamda also >1.  See Section 7.4.
%           
% Inputs:
%    L   = Order of normalized low-pass analog filter.
%          Odd order L:   Last section will contain single pole.
%          Even order L:  Trans. func. scaled in first section.
%    k   = Section number. For second order sections, k<=(L+1)/2.
%    ws  = Low edge of stopband, normalized to 1 rad/s as in (7.18).
%  lamda = Stopband parameter lambda. Gain=1/lamda at ws rad/sec.
% Outputs:  
%    d   = D(s) coefficient row vector.
%    c   = C(s) coefficient row vector.

if (L<=0) | (k<1) | (k>fix((L+1)/2))
   error('SPCBII: called with invalid (L,k) combination.');
end
if (ws<=1) | (lamda<=0)
   error('SPCBII: called with ws and/or lamda invalid.');
end
gam=(lamda+sqrt(lamda^2-1))^(1/L);
alpha=0.5*(-gam+1/gam)*sin((2*k-1)*pi/(2*L));
beta=0.5*(+gam+1/gam)*cos((2*k-1)*pi/(2*L));
sigma=(ws*alpha)/(alpha^2+beta^2);
omega=(-ws*beta)/(alpha^2+beta^2);
if (fix(L/2)~=fix((L+1)/2))&(k==(L+1)/2),
   d(1)=-sigma;
   c(1)=d(1);
   c(2)=1;
else
   scln=sigma^2+omega^2;
   scld=(ws/cos((2*k-1)*pi/(2*L)))^2;
   d(1)=scln*scld;
   d(3)=scln;
   c(1)=d(1);
   c(2)=-2*sigma*scld;
   c(3)=scld;
end
return
