function [y,px,py]=spfilt(b,a,x,px,py)
% [y,px,py]=spfilt(b,a,x,px,py)
% Filters waveform x using filter weights a and b. y=filter
% output, same length as x.  px, py are past values
% of x,y which may be =0 (scalar).  See Chapt.6, Sect. 6.3.
% Transfer function coefficients are in arrays b and a:
%
%           b(1)+b(2)*z^(-1)+...+b(N)*z^(-(N-1))
%    H(Z) = ------------------------------------
%           a(1)+a(2)*z^(-1)+...+a(M)*z^(-(M-1))
%
% Inputs:
%    b   = B(z) coefficients row vector.
%    a   = A(z) coefficients row vector with a(1) ~= 0.
%    x   = input row vector.
%    px  = row vector with past values x(-1), x(-2), etc.
%    py  = row vector with past values y(-1), y(-2), etc.
%
% Outputs:  
%    y      = filter output vector, same length as x.
%    px,py  = updated past values of x,y.

if(max(size(b,2),size(a,2))~=1 & max(size(b,1),size(a,1))~=1),
   error('SPFILT: a and b must both be row vectors or col. vectors.');
end;
a=[a 0];
if (a(1)~=1) & (a(1)~=0)
   a=a/a(1);
   b=b/a(1);
end
px=[px 0]; py=[py 0];
if(length(px)<length(b)),
   px=[px zeros(1,length(b)-length(px))];
end
if(length(py)<length(a));
   py=[0 py zeros(1,length(a)-length(py))];
end
y=zeros(1,length(x));
for k=0:length(x)-1,
   px(length(b):-1:2)=px(length(b)-1:-1:1);
   py(length(a):-1:2)=py(length(a)-1:-1:1);
   px(1)=x(k+1);
   py(1)=b*px(1:length(b))'-a(2:length(a))*py(2:length(a))'; 
   y(k+1)=py(1);
end
px=px(1:length(b));
py=py(1:length(a));
return
