function [kappa,nu]=spltcf(b,a)
% [kappa,nu]=spltcf(b,a)
% Converts transfer function coefficients from direct to
% lattice form.  Direct form H(Z) is defined by:
%
%                     b(1)+b(2)*z^(-1)+...+b(N)*z^(-(N-1))
%              H(Z) = ------------------------------------
%                      1+a(2)*z^(-1)+...+a(N)*z^(-(N-1))
%
% Inputs:
%    b      = B(z) coefficients vector of length N.
%    a      = A(z) coefficients vector of length N; a(1)=1.
% Outputs:  
%    kappa  = Feedback coefficients vector.
%    nu     = Feedforward coefficients vector.
%
% See also: splflt.

N=length(b);
if(length(a) ~= N),
   error('SPLTCF: Lengths of a and b vectors do not agree.');
end
if(a(1) ~= 1),
   error('SPLTCF: a(1) must be one in this function.')
end
work=zeros(N,3);
work(1,1)=1; work(1,3)=b(1);
work((2:N),1)=a(2:N)';
work((2:N),3)=b(2:N)';
for n=N-1:-1:1,
   work((1:n+1),2)=work(n-[0:n]+1,1);
   kappa(n)=work(n+1,1);
   if abs(kappa(n))>=1,
      error('SPLTCF: Unstable transfer function; kappa >= 1.')
   end
   work(:,1)=(work(:,1)-kappa(n)*work(:,2))/(1-kappa(n)^2);
   nu(n+1)=work(n+1,3);
   work(:,3)=work(:,3)-nu(n+1)*work(:,2);
end
nu(1)=work(1,3);
return
