function [y,px,py]=sppflt(b,a,x,px,py)
% [y,px,py]=sppflt(b,a,x,px,py)
% Same as spcflt, but parallel instead of cascade. a and b are
% dimensioned Ns x N and Ns x M, respectively, where Ns=number
% of filter sections and N-1,M-1=orders of b,a. See Section 6.5.
% Transfer function is composed of Ns sections in parallel
% with mth stage transfer function defined by:
%
%                  b(m,1)+b(m,2)*z^(-1)+...+b(m,N)*z^(-(N-1))
%           H(z) = ------------------------------------------
%                  a(m,1)+a(m,2)*z^(-1)+...+a(m,M)*z^(-(M-1))
%
% Inputs:
%    b      = B(z) coefficients. 1 section per row. Ns x N.
%    a      = A(z) coefficients. 1 section per row. Ns x M.
%    x      = input vector.
%    px,py  = past value matrices with rows as in spfilt.  
%             px=py=0 (scalar) causes zero initial conditions.
% Outputs:  
%    y      = filter output vector, same length as x.
%    px,py  = updated past values of x,y.

[Ns,N]=size(b); [Nsa,M]=size(a); Nx=length(x);
if(Ns ~= Nsa),
   error('SPPFLT: a and b size mismatch.');
end
a=[a, zeros(Ns,1)]; M=M+1;

% Add rows to px and py.
[mx,nx]=size(px); [my,ny]=size(py);
px=[px; zeros(max(1,Ns-mx),nx)];
py=[py; zeros(max(1,Ns-my),ny)];

% Add columns to px and py.
[mx,nx]=size(px); [my,ny]=size(py);
px=[px,zeros(mx,1)]; nx=nx+1;
py=[py,zeros(my,1)]; ny=ny+1;
if(nx<N)
   px=[px, zeros(mx,N-nx)];
end
if(ny<M)
   py=[py, zeros(my,M-ny)];
end
% Do the filtering one section at a time.
y=zeros(1,Nx);
for m=1:Ns,
   if (a(m,1)==0),
      error('SPCFLT: "a" matrix has a zero in column 1.');
   end
   if (a(m,1)~=1),
      a(m,:)=a(m,:)/a(m,1);
      b(m,:)=b(m,:)/b(m,1);
   end

% Do the filtering one sample at a time.
   for k=1:Nx,
      px(m,(N:-1:2))=px(m,(N-1:-1:1));
      py(m,(M:-1:2))=py(m,(M-1:-1:1));
      px(m,1)=x(k);
      py(m,1)=b(m,:)*px(m,1:N)' - a(m,2:M)*py(m,2:M)';
      y(k)=y(k)+py(m,1);
      if abs(y(k))>1e10,
         error ('SPPFLT: |output| has exceeded 1.e10.');
      end
   end
end
px=px(1:Ns,1:N-1);
py=py(1:Ns,1:M-1);
return
